@@ -5,115 +5,8 @@ import numpy as np
 from shenfun.optimization import optimizer
 from shenfun.optimization.cython import la
 from shenfun.la import SparseMatrixSolver
-from shenfun.matrixbase import TPMatrix, SpectralMatrix, extract_bc_matrices, SparseMatrix
-
-
-class FDMA(SparseMatrixSolver):
-    """4-diagonal matrix solver
-
-    Parameters
-    ----------
-    mat : SparseMatrix or list of SparseMatrices
-        4-diagonal matrix with diagonals in offsets -2, 0, 2, 4
-
-    """
-    # pylint: disable=too-few-public-methods
-
-    def __init__(self, mat):
-        SparseMatrixSolver.__init__(self, mat)
-
-        self.symmetric = self.mat.issymmetric
-        N = self.mat.shape[0]
-        self.dd = self.mat[0]*np.ones(N)*self.mat.scale
-        self.u1 = self.mat[2]*np.ones(N-2)*self.mat.scale
-        self.u2 = self.mat[4]*np.ones(N-4)*self.mat.scale
-        self.ld = self.mat[-2]*np.ones(N-2)*self.mat.scale
-
-    def perform_lu(self):
-        if self._lu is None:
-            self.FDMA_LU(self.ld, self.dd, self.u1, self.u2)
-            self._lu = 1
-        return 1
-
-    @staticmethod
-    @optimizer
-    def FDMA_LU(ld, d, u1, u2):
-        n = d.shape[0]
-        for i in range(2, n):
-            ld[i-2] = ld[i-2]/d[i-2]
-            d[i] = d[i] - ld[i-2]*u1[i-2]
-            if i < n-2:
-                u1[i] = u1[i] - ld[i-2]*u2[i-2]
-
-    @staticmethod
-    @optimizer
-    def FDMA_Solve(d, u1, u2, l, x, axis=0):
-        assert x.ndim == 1, "Use optimized version for multidimensional solve"
-        n = d.shape[0]
-        for i in range(2, n):
-            x[i] -= l[i-2]*x[i-2]
-
-        x[n-1] = x[n-1]/d[n-1]
-        x[n-2] = x[n-2]/d[n-2]
-        x[n-3] = (x[n-3] - u1[n-3]*x[n-1])/d[n-3]
-        x[n-4] = (x[n-4] - u1[n-4]*x[n-2])/d[n-4]
-        for i in range(n - 5, -1, -1):
-            x[i] = (x[i] - u1[i]*x[i+2] - u2[i]*x[i+4])/d[i]
-
-    def solve(self, b, u, axis, lu):
-        if u is not b:
-            u[:] = b
-        self.FDMA_Solve(self.dd, self.u1, self.u2, self.ld, u, axis=axis)
-        return u
-
-class TwoDMA(SparseMatrixSolver):
-    """2-diagonal matrix solver
-
-    Parameters
-    ----------
-    mat : SparseMatrix
-        2-diagonal matrix with diagonals in offsets 0, 2
-
-    """
-    def __init__(self, mat):
-        SparseMatrixSolver.__init__(self, mat)
-        N = mat.shape[0]
-        self.dd = self.mat[0]*np.ones(N)*self.mat.scale
-        self.u1 = self.mat[2]*np.ones(N-2)*self.mat.scale
-
-    @staticmethod
-    @optimizer
-    def TwoDMA_Solve(d, u1, x, axis=0):
-        assert x.ndim == 1, "Use optimized version for multidimensional solve"
-        n = d.shape[0]
-        x[n-1] = x[n-1]/d[n-1]
-        x[n-2] = x[n-2]/d[n-2]
-        for i in range(n - 3, -1, -1):
-            x[i] = (x[i] - u1[i]*x[i+2])/d[i]
-
-    def apply_constraints(self, b, constraints, axis=0):
-        if len(constraints) > 0:
-            assert len(constraints) == 1
-            assert constraints[0][0] == 0, 'Can only fix first row of TwoDMA'
-            self.dd[0] = 1
-            self.u1[0] = 0
-            if b.ndim > 1:
-                shape = b.shape
-                b = b.flatten()
-                b[0] = constraints[0][1]
-                b = b.reshape(shape)
-            else:
-                b[0] = constraints[0][1]
-        return b
-
-    def perform_lu(self):
-        return 1
-
-    def solve(self, b, u, axis, lu):
-        if u is not b:
-            u[:] = b
-        self.TwoDMA_Solve(self.dd, self.u1, u, axis=axis)
-        return u
+from shenfun.matrixbase import TPMatrix, SpectralMatrix, extract_bc_matrices,\
+    SparseMatrix, get_simplified_tpmatrices
 
 
 class ADDSolver(SparseMatrixSolver):
@@ -228,7 +121,8 @@ class ANNSolver(SparseMatrixSolver):
         sl[0] = slice(None)
         us *= j2[tuple(sl)]
         u /= A.scale
-        u[0] = constraints[0][1]/(np.pi/A.testfunction[0].domain_factor())
+        for con in constraints:
+            u[con[0]] = con[1]
         A.testfunction[0].bc.set_boundary_dofs(u, True)
         if axis > 0:
             u = np.moveaxis(u, 0, axis)
@@ -272,17 +166,6 @@ class Helmholtz:
 
         - stiffness matrix, mass matrix, scale stiffness, scale mass
 
-    Attributes
-    ----------
-        axis : int
-            The axis over which to solve for
-        neumann : bool
-            Whether or not bases are Neumann
-        bc : BoundaryValues
-            For Dirichlet problem with inhomogeneous boundary values
-
-    Variables are extracted from the matrices
-
     The solver can be used along any axis of a multidimensional problem. For
     example, if the Chebyshev basis (Dirichlet or Neumann) is the last in a
     3-dimensional TensorProductSpace, where the first two dimensions use Fourier,
@@ -341,9 +224,11 @@ class Helmholtz:
 
     """
     def __init__(self, *args):
-
-        args = list(args)
+        if len(args) == 1:
+            args = args[0]
         self.bc_mats = []
+        if isinstance(args[-1], TPMatrix):
+            args = get_simplified_tpmatrices(args)
         if isinstance(args[-1], (TPMatrix, SpectralMatrix)):
             bc_mats = extract_bc_matrices([args])
             self.tpmats = args
@@ -374,7 +259,7 @@ class Helmholtz:
         neumann = self.neumann = v.boundary_condition() == 'Neumann'
         self.axis = A.axis
         shape = [1]
-        T = A.tensorproductspace
+        T = self.T = A.tensorproductspace
         if T is not None:
             shape = list(T.shape(True))
             shape[A.axis] = 1
@@ -403,7 +288,7 @@ class Helmholtz:
     def Solve_Helmholtz(b, u, neumann, u0, u1, u2, L, axis=0):
         raise NotImplementedError
 
-    def __call__(self, b, u=None, **kw):
+    def __call__(self, b, u=None, constraints=()):
         """Solve matrix problem
 
         Parameters
@@ -424,6 +309,14 @@ class Helmholtz:
             for bc_mat in self.bc_mats:
                 b -= bc_mat.matvec(u, w0)
         self.Solve_Helmholtz(b, u, self.neumann, self.u0, self.u1, self.u2, self.L, self.axis)
+        if constraints != ():
+            z0 = self.T.local_slice()
+            paxes = np.setxor1d(range(u.ndim), self.axis)
+            is_rank_zero = np.array([z0[i].start for i in paxes]).prod()
+            if is_rank_zero == 0:
+                assert len(constraints) == 1
+                s = [0]*u.ndim
+                u[tuple(s)] = constraints[0][1]
         return u
 
     def matvec(self, v, c):
@@ -550,9 +443,13 @@ class Biharmonic:
     Note that :math:`k+l` is an array of shape (N, M, 1).
     """
     def __init__(self, *args):
+        if len(args) == 1:
+            args = args[0]
 
         args = list(args)
         self.bc_mats = []
+        if isinstance(args[-1], TPMatrix):
+            args = get_simplified_tpmatrices(args)
         if isinstance(args[-1], (TPMatrix, SpectralMatrix)):
             bc_mats = extract_bc_matrices([args])
             self.tpmats = args
@@ -577,7 +474,7 @@ class Biharmonic:
                 self.alfa *= self.A.scale
                 self.beta *= self.B.scale
         elif len(args) == 6:
-            self.a0 = a0 = args[3]
+            self.a0 = a0 = np.atleast_1d(args[3]).item()
             self.alfa = alfa = args[4]
             self.beta = beta = args[5]
 
@@ -663,195 +560,3 @@ class Biharmonic:
                                self.A[2], self.B[-4], self.B[-2], self.B[0],
                                self.B[2], self.B[4], self.axis)
         return c
-
-
-class PDMA:
-    r"""Pentadiagonal matrix solver
-
-    Pentadiagonal matrix with diagonals in offsets -4, -2, 0, 2, 4
-
-    Arising with Poisson equation and biharmonic basis u
-
-    .. math::
-
-        \alpha u'' + \beta u = f
-
-    As 4 arguments
-
-    Parameters
-    ----------
-        A : SpectralMatrix
-            Stiffness matrix
-        B : SpectralMatrix
-            Mass matrix
-        alfa : array
-        beta : array
-
-    or as dict with key/vals
-
-    Parameters
-    ----------
-        solver : str
-            Choose between implementations ('cython', 'python')
-        ASBSBmat : A
-            Stiffness matrix
-        BSBSBmat : B
-            Mass matrix
-
-    where alfa and beta must be avalable as A.scale, B.scale.
-
-    Attributes
-    ----------
-        axis : int
-            The axis over which to solve for
-
-    Variables are extracted from the matrices
-
-    """
-
-    def __init__(self, *args, **kwargs):
-        if 'ASBSBmat' in kwargs:
-            assert 'BSBSBmat' in kwargs
-            A = self.A = kwargs['ASBSBmat']
-            B = self.B = kwargs['BSBSBmat']
-            self.alfa = A.scale
-            self.beta = B.scale
-
-        elif len(args) == 4:
-            A = self.A = args[0]
-            B = self.B = args[1]
-            self.alfa = args[2]
-            self.beta = args[3]
-        else:
-            raise RuntimeError('Wrong input to PDMA solver')
-
-        self.solver = kwargs.get('solver', 'cython')
-        self.d, self.u1, self.u2 = (np.zeros_like(B[0]), np.zeros_like(B[2]),
-                                    np.zeros_like(B[4]))
-        self.l1, self.l2 = np.zeros_like(B[2]), np.zeros_like(B[4])
-        self.alfa = np.atleast_1d(self.alfa)
-        self.beta = np.atleast_1d(self.beta)
-        shape = list(self.beta.shape)
-
-        if len(shape) == 1:
-            if self.solver == 'python':
-                H = self.alfa[0]*self.A + self.beta[0]*self.B
-                self.d[:] = H[0]
-                self.u1[:] = H[2]
-                self.u2[:] = H[4]
-                self.l1[:] = H[-2]
-                self.l2[:] = H[-4]
-                self.PDMA_LU(self.l2, self.l1, self.d, self.u1, self.u2)
-
-            elif self.solver == 'cython':
-                la.LU_Helmholtz_Biharmonic_1D(self.A, self.B, self.alfa[0],
-                                              self.beta[0], self.l2, self.l1,
-                                              self.d, self.u1, self.u2)
-        else:
-
-            self.axis = A.axis
-            assert self.alfa.shape[A.axis] == 1
-            assert self.beta.shape[A.axis] == 1
-            N = A.shape[0]+4
-            self.alfa = np.broadcast_to(self.alfa, shape).copy()
-            shape[A.axis] = N
-            self.d = np.zeros(shape, float)      # Diagonal entries of U
-            self.u1 = np.zeros(shape, float)     # Diagonal+2 entries of U
-            self.l1 = np.zeros(shape, float)     # Diagonal-2 entries of U
-            self.u2 = np.zeros(shape, float)     # Diagonal+4 entries of U
-            self.l2 = np.zeros(shape, float)     # Diagonal-4 entries of U
-
-            if len(shape) == 2:
-                raise NotImplementedError
-
-            elif len(shape) == 3:
-                la.LU_Helmholtz_Biharmonic_3D(A, B, A.axis, self.alfa, self.beta, self.l2,
-                                              self.l1, self.d, self.u1, self.u2)
-
-    @staticmethod
-    def PDMA_LU(l2, l1, d, u1, u2): # pragma: no cover
-        """LU decomposition of PDM (for testing only)"""
-        n = d.shape[0]
-        m = u1.shape[0]
-        k = n - m
-
-        for i in range(n-2*k):
-            lam = l1[i]/d[i]
-            d[i+k] -= lam*u1[i]
-            u1[i+k] -= lam*u2[i]
-            l1[i] = lam
-            lam = l2[i]/d[i]
-            l1[i+k] -= lam*u1[i]
-            d[i+2*k] -= lam*u2[i]
-            l2[i] = lam
-
-        i = n-4
-        lam = l1[i]/d[i]
-        d[i+k] -= lam*u1[i]
-        l1[i] = lam
-        i = n-3
-        lam = l1[i]/d[i]
-        d[i+k] -= lam*u1[i]
-        l1[i] = lam
-
-
-    @staticmethod
-    def PDMA_Solve(l2, l1, d, u1, u2, b): # pragma: no cover
-        """Solve method for PDM (for testing only)"""
-        n = d.shape[0]
-        bc = np.full_like(b, b)
-
-        bc[2] -= l1[0]*bc[0]
-        bc[3] -= l1[1]*bc[1]
-        for k in range(4, n):
-            bc[k] -= (l1[k-2]*bc[k-2] + l2[k-4]*bc[k-4])
-
-        bc[n-1] /= d[n-1]
-        bc[n-2] /= d[n-2]
-        bc[n-3] /= d[n-3]
-        bc[n-3] -= u1[n-3]*bc[n-1]/d[n-3]
-        bc[n-4] /= d[n-4]
-        bc[n-4] -= u1[n-4]*bc[n-2]/d[n-4]
-        for k in range(n-5, -1, -1):
-            bc[k] /= d[k]
-            bc[k] -= (u1[k]*bc[k+2]/d[k] + u2[k]*bc[k+4]/d[k])
-        b[:] = bc
-
-    def __call__(self, b, u=None, **kw):
-        """Solve matrix problem
-
-        Parameters
-        ----------
-            b : array
-                Array of right hand side on entry and solution on exit unless
-                u is provided.
-            u : array
-                Output array
-
-        If b and u are multidimensional, then the axis over which to solve for
-        is determined on creation of the class.
-
-        """
-        if np.ndim(u) == 3:
-            la.Solve_Helmholtz_Biharmonic_3D_ptr(self.A.axis, b, u, self.l2,
-                                                 self.l1, self.d, self.u1,
-                                                 self.u2)
-        elif np.ndim(u) == 2:
-            la.Solve_Helmholtz_Biharmonic_2D_ptr(self.A.axis, b, u, self.l2,
-                                                 self.l1, self.d, self.u1,
-                                                 self.u2)
-        else:
-            if self.solver == 'python': # pragma: no cover
-                u[:] = b
-                self.PDMA_Solve(self.l2, self.l1, self.d, self.u1, self.u2, u)
-
-            elif self.solver == 'cython':
-                if u is b:
-                    u = np.zeros_like(b)
-                #la.Solve_Helmholtz_Biharmonic_1D(b, u, self.l2, self.l1,
-                #                                 self.d, self.u1, self.u2)
-                la.Solve_Helmholtz_Biharmonic_1D_p(b, u, self.l2, self.l1,
-                                                   self.d, self.u1, self.u2)
-
-        #u /= self.mat.scale
-        return u
