@@ -80,7 +80,8 @@ from shenfun.matrixbase import SpectralMatrix, SparseMatrix
 from shenfun.la import SparseMatrixSolver
 from shenfun.la import TDMA as generic_TDMA
 from shenfun.la import PDMA as generic_PDMA
-from .la import TwoDMA, FDMA, ADDSolver, ANNSolver
+from shenfun.la import TwoDMA, FDMA
+from .la import ADDSolver, ANNSolver
 from . import bases
 
 x = sp.symbols('x', real=True)
@@ -142,14 +143,14 @@ class BSDSDmat(SpectralMatrix):
     and :math:`\phi_j` is a Shen Dirichlet basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SD)
         ck = get_ck(test[0].N, test[0].quad)
         d = {0: np.pi/2*(ck[:-2]+ck[2:]),
              2: np.array([-np.pi/2])}
-        d[-2] = d[2]
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        d[-2] = d[2].copy()
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['cython', 'self']
 
     def get_solver(self):
@@ -189,6 +190,7 @@ class BSDSDmat(SpectralMatrix):
             self.scale_array(c, self.scale)
 
         else:
+            format = None if format in ('cython', 'self') else format
             c = super(BSDSDmat, self).matvec(v, c, format=format, axis=axis)
 
         return c
@@ -218,7 +220,7 @@ class BSDSDmatW(SpectralMatrix):
         d = {0: np.pi*((ck[:-2]+1)**2+1+dk[:-2])/8,
              2: -np.pi*(ck[:-4]+3)/8,
              4: np.pi/8}
-        d[-2] = d[2]
+        d[-2] = d[2].copy()
         d[-4] = d[4]
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
@@ -242,7 +244,7 @@ class BSNSDmat(SpectralMatrix):
     Shen Neumann basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], SD)
         N = test[0].N
@@ -253,12 +255,8 @@ class BSNSDmat(SpectralMatrix):
         d = {-2: -np.pi/2,
               0: np.pi/2.*(ck[:-2]+ck[2:]*(k/(k+2))**2)}
         d2 = -np.pi/2*(k/(k+2))**2
-        d[2] = d2[:dmax(N-2, M-2, 2)]
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
-
-    def matvec(self, v, c, format='csr', axis=0):
-        c = super(BSNSDmat, self).matvec(v, c, format=format, axis=axis)
-        return c
+        d[2] = d2[:dmax(N-2, M-2, 2)].copy()
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
 class BSDSNmat(SpectralMatrix):
@@ -278,7 +276,7 @@ class BSDSNmat(SpectralMatrix):
     Shen Neumann basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SN)
         N = test[0].N
@@ -289,7 +287,7 @@ class BSDSNmat(SpectralMatrix):
         d = {0:  np.pi/2.*(ck[:-2]+ck[2:]*(k/(k+2))**2),
              2: -np.pi/2}
         d[-2] = (-np.pi/2*(k/(k+2))**2)[:dmax(N-2, M-2, -2)]
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['cython']
 
     def matvec(self, v, c, format='cython', axis=0):
@@ -307,6 +305,7 @@ class BSDSNmat(SpectralMatrix):
             cython.Matvec.BDN_matvec1D_ptr(v, c, self[-2], self[0], self[2])
             self.scale_array(c, self.scale)
         else:
+            format = None if format in self._matvec_methods else format
             c = super(BSDSNmat, self).matvec(v, c, format=format, axis=axis)
 
         return c
@@ -327,14 +326,14 @@ class BLDLDmat(SpectralMatrix):
     and :math:`\phi_j` is a  LowerDirichlet basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], LD)
         assert isinstance(trial[0], LD)
         ck = get_ck(test[0].N, test[0].quad)
         d = {0: np.pi/2*(ck[:-1]+ck[1:]),
              1: np.array([np.pi/2]),
             -1: np.array([np.pi/2])}
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 class BSNTmat(SpectralMatrix):
     r"""Mass matrix for inner product
@@ -353,14 +352,11 @@ class BSNTmat(SpectralMatrix):
     Chebyshev basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], T)
-        SpectralMatrix.__init__(self, {}, test, trial)
+        SpectralMatrix.__init__(self, {}, test, trial, scale=scale, measure=measure)
 
-    def matvec(self, v, c, format='csr', axis=0):
-        c = super(BSNTmat, self).matvec(v, c, format=format, axis=axis)
-        return c
 
 class BSNSBmat(SpectralMatrix):
     r"""Mass matrix for inner product
@@ -379,14 +375,10 @@ class BSNSBmat(SpectralMatrix):
     Shen biharmonic basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], SB)
-        SpectralMatrix.__init__(self, {}, test, trial)
-
-    def matvec(self, v, c, format='csr', axis=0):
-        c = super(BSNSBmat, self).matvec(v, c, format=format, axis=axis)
-        return c
+        SpectralMatrix.__init__(self, {}, test, trial, scale=scale, measure=measure)
 
 
 class BTTmat(SpectralMatrix):
@@ -404,11 +396,11 @@ class BTTmat(SpectralMatrix):
 
     and :math:`T_j` is the jth order Chebyshev function of the first kind.
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], T)
         assert isinstance(trial[0], T)
         ck = get_ck(min(test[0].N, trial[0].N), test[0].quad)
-        SpectralMatrix.__init__(self, {0: np.pi/2*ck}, test, trial)
+        SpectralMatrix.__init__(self, {0: np.pi/2*ck}, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['self']
 
     def matvec(self, v, c, format='csr', axis=0):
@@ -425,6 +417,7 @@ class BTTmat(SpectralMatrix):
             c[d] = self[0][s]*v[d]
             self.scale_array(c, self.scale)
         else:
+            format = None if format in self._matvec_methods else format
             c = super(BTTmat, self).matvec(v, c, format=format, axis=axis)
 
         return c
@@ -467,7 +460,7 @@ class BSNSNmat(SpectralMatrix):
     and :math:`\phi_j` is the Shen Neumann basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], SN)
         N = test[0].N
@@ -481,12 +474,12 @@ class BSNSNmat(SpectralMatrix):
         d[2] = -np.pi/2*(k[:dp]/(k[:dp]+2))**2
         dm = dmax(N-2, M-2, -2)
         d[-2] = -np.pi/2*(k[:dm]/(k[:dm]+2))**2
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
     def get_solver(self):
         return generic_TDMA
 
-    def matvec(self, v, c, format='csr', axis=0):
+    def matvec(self, v, c, format=None, axis=0):
         # Move axis to first
         if axis > 0:
             v = np.moveaxis(v, axis, 0)
@@ -526,7 +519,7 @@ class BSDTmat(SpectralMatrix):
     :math:`\phi_k` is the Shen Dirichlet basis function and :math:`T_j` is the
     Chebyshev basis function.
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], T)
         N = test[0].N-2
@@ -535,7 +528,7 @@ class BSDTmat(SpectralMatrix):
         ck = get_ck(Q+2, test[0].quad)
         d = {0: np.pi/2*ck[:Q],
              2: -np.pi/2*ck[2:(dmax(N, M, 2)+2)]}
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
 class BTSDmat(SpectralMatrix):
@@ -554,7 +547,7 @@ class BTSDmat(SpectralMatrix):
     :math:`\phi_j` is the Shen Dirichlet basis function and :math:`T_k` is the
     Chebyshev basis function.
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], T)
         assert isinstance(trial[0], SD)
         N = test[0].N
@@ -566,7 +559,7 @@ class BTSDmat(SpectralMatrix):
         if test[0].quad == 'GL':
             d[-2] = -np.pi/2*np.ones(Q)
             d[-2][-1] *= 2
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
     def __quasi__(self, Q):
         N = self.testfunction[0].N
@@ -603,7 +596,7 @@ class BTSNmat(SpectralMatrix):
     :math:`\phi_j` is the Shen Neumann basis function and :math:`T_k` is the
     Chebyshev basis function.
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], T)
         assert isinstance(trial[0], SN)
         N = test[0].N
@@ -611,7 +604,7 @@ class BTSNmat(SpectralMatrix):
         k = np.arange(N, dtype=float)
         d = {-2: -np.pi/2*ck[2:]*((k[2:]-2)/k[2:])**2,
               0: np.pi/2*ck[:-2]}
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
     def __quasi__(self, Q):
         N = self.testfunction[0].N
@@ -649,7 +642,7 @@ class BSBSBmat(SpectralMatrix):
 
     and :math:`\psi_j` is the Shen Biharmonic basis function.
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SB)
         N = test[0].N
@@ -662,9 +655,9 @@ class BSBSBmat(SpectralMatrix):
         d2 = -((k+2)/(k+3)+(k+4)*(k+1)/((k+5)*(k+3)))*np.pi
         d[2] = d2[:dmax(N-4, M-4, 2)]
         d[4] = d4[:dmax(N-4, M-4, 4)]
-        d[-2] = d2[:dmax(N-4, M-4, -2)]
-        d[-4] = d4[:dmax(N-4, M-4, -4)]
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        d[-2] = d2[:dmax(N-4, M-4, -2)].copy()
+        d[-4] = d4[:dmax(N-4, M-4, -4)].copy()
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['cython', 'self']
 
     def get_solver(self):
@@ -708,6 +701,7 @@ class BSBSBmat(SpectralMatrix):
                                         self[2], self[4])
             self.scale_array(c, self.scale)
         else:
+            format = None if format in self._matvec_methods else format
             c = super(BSBSBmat, self).matvec(v, c, format=format, axis=axis)
 
         return c
@@ -729,7 +723,7 @@ class BSBSDmat(SpectralMatrix):
     and :math:`\phi_j` is the Shen Dirichlet basis function and :math:`\psi_k`
     the Shen Biharmonic basis function.
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SD)
         N = test[0].N-4
@@ -750,7 +744,7 @@ class BSBSDmat(SpectralMatrix):
               2: -(a[:Q]+b[:Q]*ck[(test[0].N-Q):])*np.pi/2,
               4: b[:Qp]*np.pi/2}
 
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['cython', 'self']
 
     def matvec(self, v, c, format='cython', axis=0):
@@ -781,6 +775,7 @@ class BSBSDmat(SpectralMatrix):
             cython.Matvec.BBD_matvec1D_ptr(v, c, self[-2], self[0], self[2], self[4])
             self.scale_array(c, self.scale)
         else:
+            format = None if format in self._matvec_methods else format
             c = super(BSBSDmat, self).matvec(v, c, format=format, axis=axis)
 
         return c
@@ -844,7 +839,7 @@ class BSDHHmat(SpectralMatrix):
     and :math:`\phi_j` is a Heinrichs Dirichlet basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], HH)
         ck = get_ck(test[0].N, test[0].quad)
@@ -862,7 +857,7 @@ class BSDHHmat(SpectralMatrix):
             d[2]  *= 1/((k[:-4]+3)*(k[:-4]+4))
             d[4]  *= 1/((k[:-6]+5)*(k[:-6]+6))
 
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
     def get_solver(self):
         return FDMA
@@ -886,14 +881,14 @@ class CSDSNmat(SpectralMatrix):
     For simplicity, the matrix is stored including the zero index row (k=0)
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SN)
         N = test[0].N
         k = np.arange(N-2, dtype=float)
         d = {-1: -((k[1:]-1)/(k[1:]+1))**2*(k[1:]+1)*np.pi,
               1: (k[:-1]+1)*np.pi}
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['cython']
 
     def matvec(self, v, c, format='cython', axis=0):
@@ -908,6 +903,7 @@ class CSDSNmat(SpectralMatrix):
             cython.Matvec.CDN_matvec1D_ptr(v, c, self[-1], self[1])
             self.scale_array(c, self.scale)
         else:
+            format = None if format in self._matvec_methods else format
             c = super(CSDSNmat, self).matvec(v, c, format=format, axis=axis)
 
         return c
@@ -928,14 +924,14 @@ class CSDSDmat(SpectralMatrix):
 
     and :math:`\phi_k` is the Shen Dirichlet basis function.
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SD)
         N = test[0].N
         k = np.arange(N, dtype=float)
         d = {-1: -(k[1:N-2]+1)*np.pi,
               1: (k[:(N-3)]+1)*np.pi}
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['cython', 'self']
 
     def matvec(self, v, c, format='cython', axis=0):
@@ -964,6 +960,7 @@ class CSDSDmat(SpectralMatrix):
             cython.Matvec.CDD_matvec1D_ptr(v, c, self[-1], self[1])
             self.scale_array(c, self.scale)
         else:
+            format = None if format in self._matvec_methods else format
             c = super(CSDSDmat, self).matvec(v, c, format=format, axis=axis)
 
         return c
@@ -986,25 +983,25 @@ class CSNSDmat(SpectralMatrix):
     the Shen Neumann basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], SD)
         N = test[0].N
         k = np.arange(N-2, dtype=float)
+        self._keyscale = 1
         def _getkey(i):
-            return -(1-k[:-i]**2/(k[:-i]+2)**2)*2*np.pi
+            if i == -1:
+                return -(k[1:]+1)*np.pi*self._keyscale
+            elif i == 1:
+                return -(2-k[:-1]**2/(k[:-1]+2)**2*(k[:-1]+3))*np.pi*self._keyscale
+            return -(1-k[:-i]**2/(k[:-i]+2)**2)*2*np.pi*self._keyscale
         d = dict.fromkeys(np.arange(-1, N-1, 2), _getkey)
-        d[-1] = -(k[1:]+1)*np.pi
-        d[1] = -(2-k[:-1]**2/(k[:-1]+2)**2*(k[:-1]+3))*np.pi
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
-
-    def matvec(self, v, c, format='csr', axis=0):
-        c = super(CSNSDmat, self).matvec(v, c, format=format, axis=axis)
-        #s = [slice(None),]*v.ndim
-        #if self.testfunction[0].use_fixed_gauge:
-        #    s[axis] = 0
-        #    c[tuple(s)] = 0
-        return c
+        #def _getkey(i):
+        #    return -(1-k[:-i]**2/(k[:-i]+2)**2)*2*np.pi
+        #d = dict.fromkeys(np.arange(-1, N-1, 2), _getkey)
+        #d[-1] = -(k[1:]+1)*np.pi
+        #d[1] = -(2-k[:-1]**2/(k[:-1]+2)**2*(k[:-1]+3))*np.pi
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
 class CTSDmat(SpectralMatrix):
@@ -1023,14 +1020,14 @@ class CTSDmat(SpectralMatrix):
     :math:`\phi_j` is the Shen Dirichlet basis function and :math:`T_k` is the
     Chebyshev basis function.
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], T)
         assert isinstance(trial[0], SD)
         N = test[0].N
         k = np.arange(N, dtype=float)
         d = dict.fromkeys(np.arange(-1, N-2, 2), -2*np.pi)
         d[-1] = -(k[1:N-1]+1)*np.pi
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
 class CSDTmat(SpectralMatrix):
@@ -1049,13 +1046,13 @@ class CSDTmat(SpectralMatrix):
     :math:`\phi_k` is the Shen Dirichlet basis function and :math:`T_j` is the
     Chebyshev basis function.
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], T)
         N = test[0].N
         k = np.arange(N, dtype=float)
         d = {1: np.pi*(k[:N-2]+1)}
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 class CTTmat(SpectralMatrix):
     r"""Matrix for inner product
@@ -1072,7 +1069,7 @@ class CTTmat(SpectralMatrix):
 
     :math:`T_k` is the Chebyshev basis function.
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], T)
         assert isinstance(trial[0], T)
         N = test[0].N
@@ -1081,7 +1078,7 @@ class CTTmat(SpectralMatrix):
         def _getkey(i):
             return np.pi*self._keyscale*k[i:]
         d = dict.fromkeys(np.arange(1, N, 2), _getkey)
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['cython']
 
     def matvec(self, v, c, format='cython', axis=0):
@@ -1096,6 +1093,7 @@ class CTTmat(SpectralMatrix):
             cython.Matvec.CTT_matvec(v, c)
             self.scale_array(c, self.scale*self._keyscale)
         else:
+            format = None if format in self._matvec_methods else format
             c = super(CTTmat, self).matvec(v, c, format=format, axis=axis)
         return c
 
@@ -1116,7 +1114,7 @@ class CSBSDmat(SpectralMatrix):
     :math:`\phi_j` is the Shen Dirichlet basis and :math:`\psi_k` the Shen
     Biharmonic basis function.
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SD)
         N = test[0].N
@@ -1124,7 +1122,7 @@ class CSBSDmat(SpectralMatrix):
         d = {-1: -(k[1:N-4]+1)*np.pi,
               1: 2*(k[:N-4]+1)*np.pi,
               3: -(k[:N-5]+1)*np.pi}
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['cython', 'self']
 
     def matvec(self, v, c, format='cython', axis=0):
@@ -1152,6 +1150,7 @@ class CSBSDmat(SpectralMatrix):
             cython.Matvec.CBD_matvec(v, c, self[-1], self[1], self[3])
             self.scale_array(c, self.scale)
         else:
+            format = None if format in self._matvec_methods else format
             c = super(CSBSDmat, self).matvec(v, c, format=format, axis=axis)
         return c
 
@@ -1170,7 +1169,7 @@ class CSDSBmat(SpectralMatrix):
     :math:`\phi_k` is the Shen Dirichlet basis function and :math:`\psi_j` the
     Shen Biharmonic basis function.
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SB)
         N = test[0].N
@@ -1178,7 +1177,7 @@ class CSDSBmat(SpectralMatrix):
         d = {-3: (k[3:-2]-2)*(k[3:-2]+1)/k[3:-2]*np.pi,
              -1: -2*(k[1:-3]+1)**2/(k[1:-3]+2)*np.pi,
               1: (k[:-5]+1)*np.pi}
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['cython', 'self']
 
     def matvec(self, v, c, format='cython', axis=0):
@@ -1208,6 +1207,7 @@ class CSDSBmat(SpectralMatrix):
             cython.Matvec.CDB_matvec(v, c, self[-3], self[-1], self[1])
             self.scale_array(c, self.scale)
         else:
+            format = None if format in self._matvec_methods else format
             c = super(CSDSBmat, self).matvec(v, c, format=format, axis=axis)
 
         return c
@@ -1229,7 +1229,7 @@ class ASBSBmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Biharmonic basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SB)
         N = test[0].N
@@ -1237,7 +1237,7 @@ class ASBSBmat(SpectralMatrix):
         d = {-2: 2*(k[2:]-1)*(k[2:]+2)*np.pi,
               0: -4*((k+1)*(k+2)**2)/(k+3)*np.pi,
               2: 2*(k[:-2]+1)*(k[:-2]+2)*np.pi}
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['cython', 'self']
 
     def matvec(self, v, c, format='cython', axis=0):
@@ -1266,8 +1266,8 @@ class ASBSBmat(SpectralMatrix):
         elif format == 'cython' and v.ndim == 1:
             cython.Matvec.Tridiagonal_matvec(v, c, self[-2], self[0], self[2])
             self.scale_array(c, self.scale)
-
         else:
+            format = None if format in self._matvec_methods else format
             c = super(ASBSBmat, self).matvec(v, c, format=format, axis=axis)
 
         return c
@@ -1289,7 +1289,7 @@ class ASDSDmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Dirichlet basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SD)
         N = test[0].N
@@ -1306,7 +1306,7 @@ class ASDSDmat(SpectralMatrix):
             return -self._keyscale*4*np.pi*(k[:-(i+2)]+1)
 
         d = dict.fromkeys(np.arange(0, N-2, 2), _getkey)
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['cython']
 
     def get_solver(self):
@@ -1324,6 +1324,7 @@ class ASDSDmat(SpectralMatrix):
             cython.Matvec.ADD_matvec(v, c, self[0]/self._keyscale)
             self.scale_array(c, self.scale*self._keyscale)
         else:
+            format = None if format in self._matvec_methods else format
             c = super(ASDSDmat, self).matvec(v, c, format=format, axis=axis)
 
         return c
@@ -1372,24 +1373,24 @@ class ASNSNmat(SpectralMatrix):
     and :math:`\phi_k` is the Shen Neumann basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], SN)
         N = test[0].N
         k = np.arange(N-2, dtype=float)
+        self._keyscale = 1
         def _getkey(i):
-            return -4*np.pi*(k[:-i]+i)**2*(k[:-i]+1)/(k[:-i]+2)**2
+            if i == 0:
+                return -2*np.pi*k**2*(k+1)/(k+2)*self._keyscale
+            return -4*np.pi*(k[:-i]+i)**2*(k[:-i]+1)/(k[:-i]+2)**2*self._keyscale
         d = dict.fromkeys(np.arange(0, N-2, 2), _getkey)
-        d[0] = -2*np.pi*k**2*(k+1)/(k+2)
-        d[2] = -4*np.pi*(k[:-2]+2)**2*(k[:-2]+1)/(k[:-2]+2)**2
-        d[4] = -4*np.pi*(k[:-4]+4)**2*(k[:-4]+1)/(k[:-4]+2)**2
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['numba']
 
     def get_solver(self):
         return ANNSolver
 
-    def matvec(self, v, c, format='csr', axis=0):
+    def matvec(self, v, c, format=None, axis=0):
         if format == 'numba':
             try:
                 c = numba.helmholtz.ANN_matvec(v, c, self, axis)
@@ -1445,7 +1446,7 @@ class ASBSDmat(SpectralMatrix):
     and :math:`\phi_j` is the Shen Dirichlet basis function and :math:`\psi_k`
     the Shen Biharmonic basis function.
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SD)
         N = test[0].N-4
@@ -1458,7 +1459,7 @@ class ASBSDmat(SpectralMatrix):
 
         d = {0: -2*(k+1)*(k+2)*np.pi,
              2: 2*(k[:Q]+1)*(k[:Q]+2)*np.pi}
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
 class ATTmat(SpectralMatrix):
@@ -1477,7 +1478,7 @@ class ATTmat(SpectralMatrix):
     and :math:`\psi_k` is the Chebyshev basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], T)
         assert isinstance(trial[0], T)
         N = test[0].N
@@ -1486,7 +1487,7 @@ class ATTmat(SpectralMatrix):
         def _getkey(j):
             return self._keyscale*k[j:]*(k[j:]**2-k[:-j]**2)*np.pi/2
         d = dict.fromkeys(np.arange(2, N, 2), _getkey)
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['cython']
 
     def matvec(self, v, c, format='cython', axis=0):
@@ -1501,6 +1502,7 @@ class ATTmat(SpectralMatrix):
             cython.Matvec.ATT_matvec(v, c)
             self.scale_array(c, self.scale*self._keyscale)
         else:
+            format = None if format in self._matvec_methods else format
             c = super(ATTmat, self).matvec(v, c, format=format, axis=axis)
         return c
 
@@ -1529,17 +1531,19 @@ class ATSDmat(SpectralMatrix):
     is Shen's Dirichlet basis.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], T)
         assert isinstance(trial[0], SD)
         N = test[0].N
         k = np.arange(N, dtype=float)
+        self._keyscale = 1
         def _getkey(j):
+            if j == 0:
+                return -np.pi/2*k[2:]*(k[2:]**2-k[:-2]**2)*self._keyscale
             return (k[j:-2]*(k[j:-2]**2-k[:-(j+2)]**2) -
-                    k[j+2:]*(k[j+2:]**2-k[:-(j+2)]**2))*np.pi/2.
+                    k[j+2:]*(k[j+2:]**2-k[:-(j+2)]**2))*np.pi/2.*self._keyscale
         d = dict.fromkeys(np.arange(0, N-2, 2), _getkey)
-        d[0] = -np.pi/2*k[2:]*(k[2:]**2-k[:-2]**2)
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
     def __quasi__(self, Q):
         N = self.testfunction[0].N
@@ -1564,16 +1568,18 @@ class ATSNmat(SpectralMatrix):
     is the ShenNeumann basis.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], T)
         assert isinstance(trial[0], SN)
         N = test[0].N
         k = np.arange(N, dtype=float)
+        self._keyscale = 1
         def _getkey(j):
-            return (k[j:-2]*(k[j:-2]**2-k[:-(j+2)]**2) - k[j:-2]**2/(k[j:-2]+2)*((k[j:-2]+2)**2-k[:-(j+2)]**2))*np.pi/2.
+            if j == 0:
+                return -k[:-2]**2/(k[:-2]+2)*((k[:-2]+2)**2-k[:-2]**2)*np.pi/2.*self._keyscale
+            return (k[j:-2]*(k[j:-2]**2-k[:-(j+2)]**2) - k[j:-2]**2/(k[j:-2]+2)*((k[j:-2]+2)**2-k[:-(j+2)]**2))*np.pi/2.*self._keyscale
         d = dict.fromkeys(np.arange(0, N-2, 2), _getkey)
-        d[0] = - k[:-2]**2/(k[:-2]+2)*((k[:-2]+2)**2-k[:-2]**2)*np.pi/2.
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
     def __quasi__(self, Q):
         N = self.testfunction[0].N
@@ -1731,7 +1737,7 @@ class AHHHHmat(SpectralMatrix):
     and :math:`\psi_k` is the Heinrichs Dirichlet basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], HH)
         assert isinstance(trial[0], HH)
         assert test[0].is_scaled() == trial[0].is_scaled()
@@ -1748,7 +1754,7 @@ class AHHHHmat(SpectralMatrix):
             d[2] *= 1/((k[:-2]+1)*(k[:-2]+2)*(k[:-2]+3)*(k[:-2]+4))
             d[-2]*= 1/(k[1:-1]*k[2:]*(k[2:]+1)*(k[2:]+2))
 
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 class BDUSDmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
@@ -1799,7 +1805,7 @@ class BHHHHmat(SpectralMatrix):
     and :math:`\phi_j` is a Heinrichs Dirichlet basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], HH)
         assert isinstance(trial[0], HH)
         ck = get_ck(test[0].N-2, test[0].quad)
@@ -1816,7 +1822,7 @@ class BHHHHmat(SpectralMatrix):
             d[4] *= 1/((k[:-4]+1)*(k[:-4]+2)*(k[:-4]+5)*(k[:-4]+6))
         d[-2] = d[2].copy()
         d[-4] = d[4].copy()
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
     def get_solver(self):
         return generic_PDMA
@@ -2014,7 +2020,7 @@ class SSBSBmat(SpectralMatrix):
 
     and :math:`\psi_k` is the Shen Biharmonic basis function.
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SB)
         N = test[0].N
@@ -2028,7 +2034,7 @@ class SSBSBmat(SpectralMatrix):
                 i = 8*self._keyscale*(ki[:-j]+1)*(ki[:-j]+2)*(ki[:-j]*(ki[:-j]+4)+3*(ki[j:]+2)**2)
                 return np.array(i*np.pi/(k[j:]+3))
         d = dict.fromkeys(np.arange(0, N-4, 2), _getkey)
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['cython']
 
     def matvec(self, v, c, format='cython', axis=0):
@@ -2042,8 +2048,8 @@ class SSBSBmat(SpectralMatrix):
         elif format == 'cython' and v.ndim == 1:
             cython.Matvec.SBBmat_matvec(v, c, self[0]/self._keyscale)
             self.scale_array(c, self.scale*self._keyscale)
-
         else:
+            format = None if format in self._matvec_methods else format
             c = super(SSBSBmat, self).matvec(v, c, format=format, axis=axis)
 
         return c
@@ -2066,14 +2072,14 @@ class BSDBCDmat(SpectralMatrix):
     :math:`\phi_k` is the Shen Dirichlet basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], BCD)
         d = {0: np.array([np.pi/2, np.pi/4]),
              1: np.array([np.pi/2]),
             -1: np.array([-np.pi/4, 0])}
 
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
 class BSBBCBmat(SpectralMatrix):
@@ -2093,7 +2099,7 @@ class BSBBCBmat(SpectralMatrix):
     :math:`\phi_k` is the Shen Biharmonic basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], BCB)
         d = {0: np.array([np.pi/2, np.pi*21/64, -np.pi/16, np.pi/32]),
@@ -2103,12 +2109,12 @@ class BSBBCBmat(SpectralMatrix):
             -1: np.array([-np.pi*21/64, 0, np.pi/32, 0]),
             -2: np.array([0, -np.pi/32, 0, 0]),
             -3: np.array([np.pi/32, 0, 0, 0])}
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
 class _Chebmatrix(SpectralMatrix):
-    def __init__(self, test, trial, measure=1):
-        SpectralMatrix.__init__(self, {}, test, trial, measure=measure)
+    def __init__(self, test, trial, scale=1, measure=1):
+        SpectralMatrix.__init__(self, {}, test, trial, scale=scale, measure=measure)
 
 
 class _ChebMatDict(dict):
