@@ -1,6 +1,7 @@
 import numbers
 import sympy as sp
 import numpy as np
+from shenfun.config import config
 
 class Coordinates:
     """Class for handling curvilinear coordinates
@@ -33,8 +34,10 @@ class Coordinates:
         self._hi = None
         self._b = None
         self._bt = None
+        self._e = None
         self._g = None
         self._gt = None
+        self._gn = None
         self._ct = None
         self._det_g = {True: None, False: None}
         self._sqrt_det_g = {True: None, False: None}
@@ -43,12 +46,18 @@ class Coordinates:
     def b(self):
         return self.get_covariant_basis()
 
+    @property
+    def e(self):
+        return self.get_normal_basis()
+
     @property
     def hi(self):
         return self.get_scaling_factors()
 
     @property
     def sg(self):
+        if self.is_cartesian:
+            return 1
         return self.get_sqrt_det_g(True)
 
     @property
@@ -120,6 +129,18 @@ class Coordinates:
         self._hi = hi
         return hi
 
+    def get_normal_basis(self):
+        if self._e is not None:
+            return self._e
+        b = self.b
+        e = np.zeros_like(b)
+        for i, bi in enumerate(b):
+            l = sp.sqrt(sp.simplify(np.dot(bi, bi)))
+            l = self.refine(l)
+            e[i] = bi / l
+        self._e = e
+        return e
+
     def get_covariant_basis(self):
         """Return covariant basisvectors"""
         if self._b is not None:
@@ -130,6 +151,14 @@ class Coordinates:
                 b[i, j] = rj.diff(ti, 1)
                 b[i, j] = sp.simplify(b[i, j], measure=self._measure)
                 b[i, j] = self.refine(b[i, j])
+
+        #if len(self.psi) == 2 and len(self.rv) == 3:
+        #    b[-1] = np.cross(b[0], b[1])
+        #    bl = self.refine(sp.sqrt(sp.simplify(np.dot(b[-1], b[-1]))))
+        #    b[-1] = b[-1] / bl
+        #    for j in range(len(self.rv)):
+        #        b[-1, j] = sp.simplify(b[-1, j], measure=self._measure)
+        #        b[-1, j] = self.refine(b[-1, j])
         self._b = b
         return b
 
@@ -150,6 +179,20 @@ class Coordinates:
         self._bt = bt
         return bt
 
+    def get_normal_metric_tensor(self):
+        """Return normal metric tensor"""
+        if self._gn is not None:
+            return self._gn
+        gn = np.zeros((len(self.psi), len(self.psi)), dtype=object)
+        e = self.e
+        for i in range(len(self.psi)):
+            for j in range(len(self.psi)):
+                gn[i, j] = sp.simplify(np.dot(e[i], e[j]).expand(), measure=self._measure)
+                gn[i, j] = self.refine(gn[i, j])
+
+        self._gn = gn
+        return gn
+
     def get_covariant_metric_tensor(self):
         """Return covariant metric tensor"""
         if self._g is not None:
@@ -191,6 +234,17 @@ class Coordinates:
         self._ct = ct
         return ct
 
+    def get_metric_tensor(self, kind='normal'):
+        if kind == 'covariant':
+            gij = self.get_covariant_metric_tensor()
+        elif kind == 'contravariant':
+            gij = self.get_contravariant_metric_tensor()
+        elif kind == 'normal':
+            gij = self.get_normal_metric_tensor()
+        else:
+            raise NotImplementedError
+        return gij
+
     def refine(self, sc):
         sc = sp.refine(sc, self._assumptions)
         for a, b in self._replace:
@@ -220,11 +274,15 @@ class Coordinates:
         self._psi = tuple([p.subs(s0, s1) for p in self._psi])
         self._rv = tuple([r.subs(s0, s1) for r in self._rv])
 
-    def latex_basis_vectors(self, symbol_names=None, covariant=True, replace=None):
-        if covariant:
+    def latex_basis_vectors(self, symbol_names=None, replace=None, kind=None):
+        if kind is None:
+            kind = config['basisvectors']
+        if kind == 'covariant':
             b = self.get_covariant_basis()
-        else:
+        elif kind == 'contravariant':
             b = self.get_contravariant_basis()
+        else:
+            b = self.get_normal_basis()
         psi = self.psi
         symbols = {p: str(p) for p in psi}
         if symbol_names is not None:
@@ -232,11 +290,12 @@ class Coordinates:
 
         k = {0: '\\mathbf{i}', 1: '\\mathbf{j}', 2: '\\mathbf{k}'}
         m = ' '
+        bl = 'e' if kind == 'normal' else 'b'
         for i, p in enumerate(psi):
-            if covariant:
-                m += '\\mathbf{b}_{%s} ='%(symbols[p])
+            if kind in ('covariant', 'normal'):
+                m += '\\mathbf{%s}_{%s} ='%(bl, symbols[p])
             else:
-                m += '\\mathbf{b}^{%s} ='%(symbols[p])
+                m += '\\mathbf{%s}^{%s} ='%(bl, symbols[p])
             for j in range(b.shape[1]):
                 if b[i, j] == 1:
                     m += (k[j]+'+')
@@ -256,3 +315,4 @@ class Coordinates:
             m += ' \\\\ '
         m += ' '
         return r'%s'%(m)
+
