@@ -1,27 +1,20 @@
 from numbers import Number, Integral
+from itertools import product
 import copy
 from scipy.special import sph_harm, erf, airy
 import numpy as np
 import sympy as sp
+from shenfun.config import config
 from shenfun.optimization.cython import evaluate
 from mpi4py_fft import DistArray
 
 __all__ = ('Expr', 'BasisFunction', 'TestFunction', 'TrialFunction', 'Function',
-           'Array', 'FunctionSpace', 'Basis')
+           'Array', 'FunctionSpace')
 
 # Define some special functions required for spherical harmonics
 cot = lambda x: 1/np.tan(x)
 Ynm = lambda n, m, x, y : sph_harm(m, n, y, x)
 airyai = lambda x: airy(x)[0]
-printwarning = True
-
-def Basis(*args, **kwargs): #pragma: no cover
-    global printwarning
-    import warnings
-    if printwarning:
-        warnings.warn("Basis() is deprecated; use FunctionSpace().", FutureWarning)
-        printwarning = False
-    return FunctionSpace(*args, **kwargs)
 
 def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
                   domain=None, scaled=None, padding_factor=1, basis=None,
@@ -458,8 +451,6 @@ class Expr:
         if indices is None:
             self._indices = (basis.offset()+np.arange(self.function_space().num_components())[:, np.newaxis]).tolist()
 
-        #assert np.prod(self._scales.shape) == self.num_terms()*self.num_components()
-
     def basis(self):
         """Return basis of Expr"""
         return self._basis
@@ -519,7 +510,8 @@ class Expr:
     @property
     def dimensions(self):
         """Return ndim of Expr"""
-        return len(self._terms[0][0])
+        return self.function_space().dimensions
+        #return len(self._terms[0][0])
 
     def index(self):
         if self.num_components() == 1:
@@ -529,13 +521,15 @@ class Expr:
     def tolatex(self, symbol_names=None, funcname='u', replace=None):
         s = ""
         x = 'xyzrst'
-        symbols = {k: k for k in x}
+        xx = list(product(x[:self.dimensions], x[:self.dimensions]))
+        symbols = {k: k for k in x[:self.dimensions]}
         if symbol_names is not None:
             symbols = {str(k): val for k, val in symbol_names.items()}
 
+        bl = 'e' if config['basisvectors'] == 'normal' else 'b'
         for i, vec in enumerate(self.terms()):
             if self.num_terms()[i] == 0 and self.num_components() > 1:
-                s += '0 \\mathbf{b}_{%s} \\\\ +'%(symbols[x[i]])
+                s += '0 \\mathbf{%s}_{%s} \\\\ +'%(bl, symbols[x[i]])
                 continue
 
             if self.num_components() > 1:
@@ -544,6 +538,7 @@ class Expr:
             for j, term in enumerate(vec):
                 sc = self.scales()[i][j]
                 k = self.indices()[i][j]
+                kk = k % self.dimensions
                 if not sc == 1:
                     if replace is not None:
                         for repl in replace:
@@ -556,13 +551,15 @@ class Expr:
                         scp = "\\left( %s \\right)"%(scp)
                     elif scp.startswith('-'):
                         s = s.rstrip('+')
+                    if scp == '-1':
+                        scp = '-'
                     s += scp
                 t = np.array(term)
                 cmp = funcname
-                #if self.num_components() > 1:
-
-                if self.tensor_rank > 0:
-                    cmp = funcname + '^{%s}'%(symbols[x[k]])
+                if self.tensor_rank == 1:
+                    cmp = funcname + '^{%s}'%(symbols[x[kk]])
+                elif self.tensor_rank == 2:
+                    cmp = funcname + '^{%s%s}'%(tuple([symbols[xj] for xj in xx[k]]))
                 if np.sum(t) == 0:
                     s += cmp
                 else:
@@ -576,7 +573,11 @@ class Expr:
                 s += '+'
             if self.num_components() > 1:
                 s = s.rstrip('+')
-                s += "\\right) \\mathbf{b}_{%s} \\\\"%(symbols[x[i]])
+                if self.expr_rank() == 1:
+                    s += "\\right) \\mathbf{%s}_{%s} \\\\"%(bl, symbols[x[i]])
+                elif self.expr_rank() == 2:
+                    ij = tuple([bl]+[symbols[xj] for xj in xx[i]])
+                    s += "\\right) \\mathbf{%s}_{%s} \\mathbf{b}_{%s} \\\\"%(ij)
                 s += '+'
 
         return r"""%s"""%(s.rstrip('+'))
@@ -733,15 +734,18 @@ class Expr:
 
         return output_array
 
+    @property
+    def is_basis_function(self):
+        return np.all(np.array([q for l in self.terms() for s in l for q in s]) == 0)
+
     def __getitem__(self, i):
         if i >= self.dimensions:
             raise IndexError
 
         basis = self._basis
-        if basis.function_space().is_composite_space > 0:
+        if self.expr_rank() == self.tensor_rank and self.is_basis_function:
             basis = self._basis[i]
-        else:
-            basis = self._basis
+
         if self.expr_rank() == 1:
             return Expr(basis,
                         [copy.deepcopy(self._terms[i])],
@@ -757,52 +761,65 @@ class Expr:
         else:
             raise NotImplementedError
 
+    def get_contravariant_component(self, i, j=None):
+        """Return contravariant vector component
+
+        Parameters
+        ----------
+        i : int
+            First vector component
+        j : int, optional
+            Second component of tensor
+
+        Note
+        ----
+        The contravariant vector components are the components to the
+        covariant basisvectors. If the basis vectors are normalize, i.e.,
+        if the setting is config['basisvectors'] == 'normal', then the
+        normal vector components must be scaled in order to get to the
+        contravariant components.
+        """
+        if j is None:
+            if i >= self.dimensions:
+                raise IndexError
+
+            if config['basisvectors'] == 'normal':
+                return self[i]*(1/self.function_space().coors.hi[i])
+            assert config['basisvectors'] == 'covariant'
+            return self[i]
+        assert isinstance(j, int)
+        if i >= self.dimensions or j >= self.dimensions:
+            raise IndexError
+
+        if config['basisvectors'] == 'normal':
+            hi = self.function_space().coors.hi
+            return self[i][j]*(1/(hi[i]*hi[j]))
+        assert config['basisvectors'] =='covariant'
+        return self[i][j]
+
     def __mul__(self, a):
-        sc = np.array(self.scales())
-        if self.expr_rank() == 0:
-            sc = sc*sp.sympify(a)
-
-        else:
-            if isinstance(a, tuple):
-                assert len(a) == self.num_components()
-                for i in range(self.num_components()):
-                    sc[i] = sc[i]*sp.sympify(a[i])
-
-            else:
-                sc = sc*sp.sympify(a)
-
-        for i in range(sc.shape[0]):
-            for j in range(sc.shape[1]):
-                sc[i, j] = self.function_space().coors.refine(sc[i, j])
-                sc[i, j] = sp.simplify(sc[i, j], measure=self.function_space().coors._measure)
-
-        return Expr(self._basis, copy.deepcopy(self._terms), sc.tolist(), copy.deepcopy(self._indices))
+        sc = copy.deepcopy(self.scales())
+        coors = self.function_space().coors
+        for i, vec in enumerate(sc):
+            ai = a[i] if isinstance(a, tuple) else a
+            ai = sp.sympify(ai)
+            for j in range(len(vec)):
+                vec[j] = coors.refine(vec[j]*ai)
+                vec[j] = sp.simplify(vec[j], measure=self.function_space().coors._measure)
+        return Expr(self._basis, copy.deepcopy(self._terms), sc, copy.deepcopy(self._indices))
 
     def __rmul__(self, a):
         return self.__mul__(a)
 
     def __imul__(self, a):
-        sc = np.array(self.scales())
-        if self.expr_rank() == 0:
-            sc = sc*sp.sympify(a)
-
-        else:
-            if isinstance(a, tuple):
-                assert len(a) == self.dimensions
-                for i in range(self.dimensions):
-                    sc[i] = sc[i]*sp.sympify(a[i])
-
-            else:
-                sc = sc*sp.sympify(a)
-
+        sc = self.scales()
         coors = self.function_space().coors
-        for i in range(sc.shape[0]):
-            for j in range(sc.shape[1]):
-                sc[i, j] = coors.refine(sc[i, j])
-                sc[i, j] = sp.simplify(sc[i, j], measure=coors._measure)
-
-        self._scales = sc.tolist()
-
+        for i, vec in enumerate(sc):
+            ai = a[i] if isinstance(a, tuple) else a
+            ai = sp.sympify(ai)
+            for j in range(len(vec)):
+                vec[j] = coors.refine(vec[j])
+                vec[j] = sp.simplify(vec[j]*ai, measure=coors._measure)
         return self
 
     def __add__(self, a):
@@ -970,18 +987,15 @@ class BasisFunction:
         :class:`.SpectralBase`
     index : int
         Local component of basis in :class:`.CompositeSpace`
-    basespace : The base :class:`.CompositeSpace` if space is a
-        subspace.
     offset : int
         The number of scalar spaces (i.e., :class:`.TensorProductSpace`es)
         ahead of this space
     base : The base :class:`BasisFunction`
     """
 
-    def __init__(self, space, index=0, basespace=None, offset=0, base=None):
+    def __init__(self, space, index=0, offset=0, base=None):
         self._space = space
         self._index = index
-        self._basespace = basespace
         self._offset = offset
         self._base = base
 
@@ -998,11 +1012,6 @@ class BasisFunction:
         """Return function space of BasisFunction"""
         return self._space
 
-    @property
-    def basespace(self):
-        """Return base space"""
-        return self._basespace if self._basespace is not None else self._space
-
     @property
     def base(self):
         """Return base """
@@ -1036,13 +1045,12 @@ class BasisFunction:
 
     def __getitem__(self, i):
         assert self.function_space().is_composite_space
-        basespace = self.basespace
         base = self.base
         space = self._space[i]
         offset = self._offset
         for k in range(i):
             offset += self._space[k].num_components()
-        t0 = self.__class__(space, 0, basespace, offset, base)
+        t0 = self.__class__(space, 0, offset, base)
         return t0
 
     def __mul__(self, a):
@@ -1080,16 +1088,14 @@ class TestFunction(BasisFunction):
     space: :class:`TensorProductSpace` or :class:`CompositeSpace`
     index: int, optional
         Component of basis in :class:`.CompositeSpace`
-    basespace : The base :class:`.CompositeSpace` if space is a
-        subspace.
     offset : int
         The number of scalar spaces (i.e., :class:`.TensorProductSpace`es)
         ahead of this space
     base : The base :class:`TestFunction`
     """
 
-    def __init__(self, space, index=0, basespace=None, offset=0, base=None):
-        BasisFunction.__init__(self, space, index, basespace, offset, base)
+    def __init__(self, space, index=0, offset=0, base=None):
+        BasisFunction.__init__(self, space, index, offset, base)
 
     @property
     def argument(self):
@@ -1103,15 +1109,13 @@ class TrialFunction(BasisFunction):
     space: :class:`TensorProductSpace` or :class:`CompositeSpace`
     index: int, optional
         Component of basis in :class:`.CompositeSpace`
-    basespace : The base :class:`.CompositeSpace` if space is a
-        subspace.
     offset : int
         The number of scalar spaces (i.e., :class:`.TensorProductSpace`es)
         ahead of this space
     base : The base :class:`TrialFunction`
     """
-    def __init__(self, space, index=0, basespace=None, offset=0, base=None):
-        BasisFunction.__init__(self, space, index, basespace, offset, base)
+    def __init__(self, space, index=0, offset=0, base=None):
+        BasisFunction.__init__(self, space, index, offset, base)
 
     @property
     def argument(self):
@@ -1197,7 +1201,7 @@ class ShenfunBaseArray(DistArray):
             assert len(buffer) == len(space.flatten())
             sympy_buffer = buffer
             buffer = Array(space)
-            dtype = space.forward.input_array.dtype
+            adtype = space.forward.input_array.dtype
             if cls.__name__ == 'Function':
                 buff = Function(space)
             mesh = space.local_mesh(True)
@@ -1210,7 +1214,7 @@ class ShenfunBaseArray(DistArray):
                     for sym in sym0:
                         j = 'xyzrs'.index(str(sym))
                         m.append(mesh[j])
-                    buffer.v[i] = sp.lambdify(sym0, buf0, modules=['numpy', {'airyai': airyai, 'cot': cot, 'Ynm': Ynm, 'erf': erf}])(*m).astype(dtype)
+                    buffer.v[i] = sp.lambdify(sym0, buf0, modules=['numpy', {'airyai': airyai, 'cot': cot, 'Ynm': Ynm, 'erf': erf}])(*m).astype(adtype)
                 else:
                     raise NotImplementedError
 
