@@ -107,7 +107,7 @@ class BLLmat(SpectralMatrix):
     and :math:`L_k` is the Legendre basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], L)
         assert isinstance(trial[0], L)
         N = test[0].N
@@ -115,21 +115,21 @@ class BLLmat(SpectralMatrix):
         d = {0: 2./(2.*k+1)}
         if test[0].quad == 'GL':
             d[0][-1] = 2./(N-1)
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
-    def solve(self, b, u=None, axis=0, constraints=()):
-        s = self.trialfunction[0].slice()
-        if u is None:
-            u = b
-        else:
-            assert u.shape == b.shape
-        sl = [np.newaxis]*u.ndim
-        sl[axis] = s
-        sl = tuple(sl)
-        ss = self.trialfunction[0].sl[s]
-        d = (1./self.scale)/self[0]
-        u[ss] = b[ss]*d[sl]
-        return u
+    #def solve(self, b, u=None, axis=0, constraints=()):
+    #    s = self.trialfunction[0].slice()
+    #    if u is None:
+    #        u = b
+    #    else:
+    #        assert u.shape == b.shape
+    #    sl = [np.newaxis]*u.ndim
+    #    sl[axis] = s
+    #    sl = tuple(sl)
+    #    ss = self.trialfunction[0].sl[s]
+    #    d = (1./self.scale)/self[0]
+    #    u[ss] = b[ss]*d[sl]
+    #    return u
 
 
 class BSDSDmat(SpectralMatrix):
@@ -148,7 +148,7 @@ class BSDSDmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SD)
         N = test[0].N
@@ -163,8 +163,8 @@ class BSDSDmat(SpectralMatrix):
             d[0] /= (4*k+6)
             d[-2] /= (np.sqrt(4*k[2:]+6)*np.sqrt(4*k[:-2]+6))
 
-        d[2] = d[-2]
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        d[2] = d[-2].copy()
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
     def get_solver(self):
         return TDMA
@@ -185,7 +185,7 @@ class BSNSNmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre Neumann basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], SN)
         N = test[0].N
@@ -196,8 +196,8 @@ class BSNSNmat(SpectralMatrix):
              2: -d0[2:]*alpha[:-2]}
         if test[0].quad == 'GL':
             d[0][-1] = d0[-1] + alpha[-1]**2*2./(N-1)
-        d[-2] = d[2]
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        d[-2] = d[2].copy()
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         #self.solve = neumann_TDMA(self)
 
 
@@ -217,7 +217,7 @@ class BSBSBmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre Biharmonic basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SB)
         N = test[0].N
@@ -230,9 +230,9 @@ class BSBSBmat(SpectralMatrix):
         d = {0: ek[:-4] + hk[:-4]**2*ek[2:-2] + gk[:-4]**2*ek[4:],
              2: hk[:-6]*ek[2:-4] + gk[:-6]*hk[2:-4]*ek[4:-2],
              4: gk[:-8]*ek[4:-4]}
-        d[-2] = d[2]
-        d[-4] = d[4]
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        d[-2] = d[2].copy()
+        d[-4] = d[4].copy()
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
     def get_solver(self):
         return PDMA
@@ -253,7 +253,7 @@ class BBFBFmat(SpectralMatrix):
     and :math:`\psi_k` is the BeamFixedFree Biharmonic basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], BF)
         assert isinstance(trial[0], BF)
         N = test[0].N
@@ -275,7 +275,7 @@ class BBFBFmat(SpectralMatrix):
         if test[0].quad == 'GL':
             k = N-5
             d[0][-1] = 2/(2*k+1)+f1(k)**2*2/(2*k+3)+f2(k)**2*2/(2*k+5)+f3(k)**2*2/(2*k+7)+f4(k)**2*2/(N-1)
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
 class BSDLmat(SpectralMatrix):
@@ -294,7 +294,7 @@ class BSDLmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], L)
         N = test[0].N
@@ -306,7 +306,7 @@ class BSDLmat(SpectralMatrix):
              0: 2./(2.*k[:-2]+1)*sc[:-2]}
         if test[0].quad == 'GL':
             d[2][-1] = -2./(N-1)*sc[N-3]
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
 class BLSDmat(SpectralMatrix):
@@ -325,7 +325,7 @@ class BLSDmat(SpectralMatrix):
     and :math:`\psi_j` is the Shen Legendre Dirichlet basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], L)
         assert isinstance(trial[0], SD)
         N = test[0].N
@@ -338,7 +338,7 @@ class BLSDmat(SpectralMatrix):
              0: 2./(2.*k[:-2]+1)*sc[:-2]}
         if test[0].quad == 'GL':
             d[-2][-1] = -2./(N-1)*sc[-3]
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 class BDNDNmat(SpectralMatrix):
     r"""Mass matrix for inner product
@@ -356,7 +356,7 @@ class BDNDNmat(SpectralMatrix):
     and :math:`\psi_k` is a mixed Legendre Dirichlet/Neumann basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], DN)
         assert isinstance(trial[0], DN)
         N = test[0].N
@@ -374,7 +374,7 @@ class BDNDNmat(SpectralMatrix):
             k = N-3
             d[0][-1] = 2/(2*k+1) + 2*((2*k+3)/(k+2))/(k+2)**3 + 2*((k+1)/(k+2))**4/(N-1)
 
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
 class ASDSDmat(SpectralMatrix):
@@ -496,8 +496,8 @@ class ASNSNmat(SpectralMatrix):
         us[:] = bs*d[tuple(sl)]
         u /= self.scale
         self.testfunction[0].bc.set_boundary_dofs(u, True)
-        if self.testfunction[0].use_fixed_gauge:
-            u[0] = self.testfunction[0].mean/(2/self.testfunction[0].domain_factor())
+        for con in constraints:
+            u[con[0]] = con[1]
         if axis > 0:
             u = np.moveaxis(u, 0, axis)
             if u is not b:
@@ -530,7 +530,7 @@ class ASBSBmat(SpectralMatrix):
         gk = (2*k+3)/(2*k+7)
         d = {0: 2*(2*k+3)*(1+gk),
              2: -2*(2*k[:-2]+3)}
-        d[-2] = d[2]
+        d[-2] = d[2].copy()
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 class ADNDNmat(SpectralMatrix):
@@ -664,7 +664,7 @@ class GLLmat(SpectralMatrix):
     and :math:`L_k` is the Legendre basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], L)
         assert isinstance(trial[0], L)
         N = test[0].N
@@ -679,7 +679,7 @@ class GLLmat(SpectralMatrix):
         else:
             d = dict.fromkeys(-np.arange(2, N, 2), _getkey)
 
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['cython']
 
     def matvec(self, v, c, format='cython', axis=0):
@@ -695,6 +695,7 @@ class GLLmat(SpectralMatrix):
             cython.Matvec.GLL_matvec(v, c)
             self.scale_array(c, self.scale*self._keyscale)
         else:
+            format = None if format in self._matvec_methods else format
             c = super(GLLmat, self).matvec(v, c, format=format, axis=axis)
         return c
 
@@ -714,13 +715,13 @@ class SSBSBmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre Biharmonic basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SB)
         N = test[0].N
         k = np.arange(N-4, dtype=float)
         d = {0: 2*(2*k+3)**2*(2*k+5)}
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
 class CLLmat(SpectralMatrix):
@@ -739,7 +740,7 @@ class CLLmat(SpectralMatrix):
     and :math:`\psi_k` is the orthogonal Legendre basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], L)
         assert isinstance(trial[0], L)
         N = test[0].N
@@ -748,7 +749,7 @@ class CLLmat(SpectralMatrix):
             return 2*self._keyscale
 
         d = dict.fromkeys(np.arange(1, N, 2), _getkey)
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self._matvec_methods += ['cython', 'self']
 
     def matvec(self, v, c, format='self', axis=0):
@@ -776,6 +777,7 @@ class CLLmat(SpectralMatrix):
             cython.Matvec.CLL_matvec(v, c)
             self.scale_array(c, self.scale*self._keyscale)
         else:
+            format = None if format in self._matvec_methods else format
             c = super(CLLmat, self).matvec(v, c, format=format, axis=axis)
         return c
 
@@ -795,7 +797,7 @@ class CLLmatT(SpectralMatrix):
     and :math:`\psi_k` is the orthogonal Legendre basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], L)
         assert isinstance(trial[0], L)
         N = test[0].N
@@ -804,7 +806,7 @@ class CLLmatT(SpectralMatrix):
             return 2*self._keyscale
 
         d = dict.fromkeys(-np.arange(1, N, 2), _getkey)
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
 class CLSDmat(SpectralMatrix):
@@ -823,7 +825,7 @@ class CLSDmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], L)
         assert isinstance(trial[0], SD)
         N = test[0].N
@@ -831,7 +833,7 @@ class CLSDmat(SpectralMatrix):
         if trial[0].is_scaled():
             k = np.arange(N-2, dtype=float)
             d[-1] = -2. / np.sqrt(4*k+6)
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
 class CSDLmat(SpectralMatrix):
@@ -850,7 +852,7 @@ class CSDLmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], L)
         N = test[0].N
@@ -858,7 +860,7 @@ class CSDLmat(SpectralMatrix):
         if test[0].is_scaled():
             k = np.arange(N-2, dtype=float)
             d[1] = -2. / np.sqrt(4*k+6)
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
 class CSDSDmat(SpectralMatrix):
@@ -1163,7 +1165,7 @@ class BUDUDmat(SpectralMatrix):
     and :math:`\psi_k` is the Legendre UpperDirichlet basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], UD)
         assert isinstance(trial[0], UD)
         N = test[0].N
@@ -1174,8 +1176,8 @@ class BUDUDmat(SpectralMatrix):
         if test[0].quad == 'GL':
             d[0][-1] = 2./(2*(N-2)+1) + 2./(N-1)
 
-        d[1] = d[-1]
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        d[1] = d[-1].copy()
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 class BUDUDrp1mat(SpectralMatrix):
     r"""Matrix for inner product
@@ -1202,8 +1204,8 @@ class BUDUDrp1mat(SpectralMatrix):
         d = {0: 4*(k+1)/(2*k+1)/(2*k+3),
              1: 4/(2*k[:-1]+1)/(2*k[:-1]+3)/(2*k[:-1]+5),
              2: -2*(k[:-2]+2)/(2*k[:-2]+3)/(2*k[:-2]+5)}
-        d[-1] = d[1]
-        d[-2] = d[2]
+        d[-1] = d[1].copy()
+        d[-2] = d[2].copy()
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
@@ -1259,9 +1261,9 @@ class BSDSDrp1mat(SpectralMatrix):
              1: 2/(2*k[:-1]+1)/(2*k[:-1]+5) + 2*(k[:-1]+3)/(2*k[:-1]+5)/(2*k[:-1]+7),
              2: -2/(2*k[:-2]+5),
              3: -2*(k[:-3]+3)/(2*k[:-3]+5)/(2*k[:-3]+7)}
-        d[-1] = d[1]
-        d[-2] = d[2]
-        d[-3] = d[3]
+        d[-1] = d[1].copy()
+        d[-2] = d[2].copy()
+        d[-3] = d[3].copy()
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
@@ -1282,7 +1284,7 @@ class BSDBCDmat(SpectralMatrix):
     :math:`\phi_k` is the Shen Dirichlet basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], BCD)
         N = test[0].N
@@ -1296,7 +1298,7 @@ class BSDBCDmat(SpectralMatrix):
                  1: np.array([1./np.sqrt(6.)]),
                  -1: np.array([-1./3./np.sqrt(10.), 0])}
 
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
 class BSBBCBmat(SpectralMatrix):
@@ -1316,7 +1318,7 @@ class BSBBCBmat(SpectralMatrix):
     :math:`\phi_k` is the Shen Biharmonic basis function.
 
     """
-    def __init__(self, test, trial, measure=1):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], BCB)
         N = test[0].N
@@ -1329,12 +1331,12 @@ class BSBBCBmat(SpectralMatrix):
             -2: np.array([0, -1/35, 0, 0]),
             -3: np.array([1/35, 0, 0, 0])}
 
-        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
 class _Legmatrix(SpectralMatrix):
-    def __init__(self, test, trial, measure=1):
-        SpectralMatrix.__init__(self, {}, test, trial, measure=measure)
+    def __init__(self, test, trial, scale=1, measure=1):
+        SpectralMatrix.__init__(self, {}, test, trial, scale=scale, measure=measure)
 
 
 class _LegMatDict(dict):
