@@ -779,15 +779,6 @@ class SpectralBase:
     def has_nonhomogeneous_bcs(self):
         return False
 
-    @property
-    def use_fixed_gauge(self):
-        """Return True if space (or TensorProductSpace this space belongs to)
-        contains only Neumann boundary conditions"""
-        T = self.tensorproductspace
-        if T:
-            return T.use_fixed_gauge
-        return self.boundary_condition() == 'Neumann'
-
     @property
     def rank(self):
         """Return rank of function space
@@ -895,11 +886,11 @@ class SpectralBase:
         if measure == 1:
             return wj
 
-        if isinstance(measure, Number):
-            wj *= measure
+        s = sp.sympify(measure).free_symbols
+        if isinstance(measure, Number) or len(s) == 0:
+            wj *= float(measure)
             return wj
 
-        s = measure.free_symbols
         assert len(s) == 1
         s = s.pop()
         xj = sp.lambdify(s, measure)(xm)
@@ -972,7 +963,7 @@ class SpectralBase:
             d['bc'] = tuple(self.bc.bc)
         if hasattr(self, '_scaled'):
             d['scaled'] = self._scaled
-        for key in ('alpha', 'beta', 'mean'):
+        for key in ('alpha', 'beta'):
             if hasattr(self, key):
                 d[key] = object.__getattribute__(self, key)
         d.update(kwargs)
@@ -1003,7 +994,7 @@ class SpectralBase:
             d['bc'] = tuple(self.bc.bc)
         if hasattr(self, '_scaled'):
             d['scaled'] = self._scaled
-        for key in ('alpha', 'beta', 'mean'):
+        for key in ('alpha', 'beta'):
             if hasattr(self, key):
                 d[key] = object.__getattribute__(self, key)
         d.update(kwargs)
@@ -1024,7 +1015,6 @@ class SpectralBase:
             - padding_factor
             - dealias_direct
             - coorrdinates
-            - mean
             - bc
             - scaled
 
@@ -1035,11 +1025,6 @@ class SpectralBase:
         SpectralBase
             Space not planned for a :class:`.TensorProductSpace`
 
-        Note
-        ----
-        Default is to return a basic space not intended for dealiasing.
-        So the `planning_factor` is set to 1 and `dealias_direct` is
-        False. This can be overruled by keyword arguments.
         """
         d = dict(quad=self.quad,
                  domain=self.domain,
@@ -1051,7 +1036,34 @@ class SpectralBase:
             d['bc'] = tuple(self.bc.bc)
         if hasattr(self, '_scaled'):
             d['scaled'] = self._scaled
-        for key in ('alpha', 'beta', 'mean'):
+        for key in ('alpha', 'beta'):
+            if hasattr(self, key):
+                d[key] = object.__getattribute__(self, key)
+        d.update(kwargs)
+        return self.__class__(self.N, **d)
+
+    def get_homogeneous(self, **kwargs):
+        """Return space (otherwise as self) with homogeneous boundary conditions
+
+        Parameters
+        ----------
+        kwargs : keyword arguments
+            Any keyword arguments used in the creation of the bases.
+
+        Returns
+        -------
+        SpectralBase
+            A new space with homogeneous boundary conditions, otherwise as self.
+        """
+        d = dict(quad=self.quad,
+                 domain=self.domain,
+                 dtype=self.dtype,
+                 padding_factor=self.padding_factor,
+                 dealias_direct=self.dealias_direct,
+                 coordinates=self.coors.coordinates)
+        if hasattr(self, '_scaled'):
+            d['scaled'] = self._scaled
+        for key in ('alpha', 'beta'):
             if hasattr(self, key):
                 d[key] = object.__getattribute__(self, key)
         d.update(kwargs)
@@ -1372,9 +1384,12 @@ def inner_product(test, trial, measure=1):
         x0 = x0.pop()
         x = sp.symbols('x', real=x0.is_real, positive=x0.is_positive)
         measure = measure.subs(x0, x)
-        if measure.subs(x, (test[0].domain[0]+test[0].domain[1])/2).evalf() < 0:
-            sc = -1
-            measure *= sc
+        try:
+            if measure.subs(x, (test[0].domain[0]+test[0].domain[1])/2).evalf() < 0:
+                sc = -1
+                measure *= sc
+        except TypeError:
+            pass
         if measure.is_polynomial():
             measure = sp.simplify(measure)
         key = key + (test[0].domain, measure)
@@ -1390,13 +1405,13 @@ class FuncWrap:
 
     # pylint: disable=too-few-public-methods, missing-docstring
 
-    __slots__ = ('_func', '_input_array', '_output_array')
+    __slots__ = ('__doc__', '_func', '_input_array', '_output_array')
 
     def __init__(self, func, input_array, output_array):
         object.__setattr__(self, '_func', func)
         object.__setattr__(self, '_input_array', input_array)
         object.__setattr__(self, '_output_array', output_array)
-        #object.__setattr__(self, '__doc__', func.__doc__)
+        object.__setattr__(self, '__doc__', func.__doc__)
 
     @property
     def input_array(self):
@@ -1418,7 +1433,7 @@ class Transform(FuncWrap):
     # pylint: disable=too-few-public-methods
 
     __slots__ = ('_xfftn', '_input_array', '_output_array',
-                 '_tmp_array')
+                 '_tmp_array', '__doc__')
 
     def __init__(self, func, xfftn, input_array, tmp_array, output_array):
         FuncWrap.__init__(self, func, input_array, output_array)
