@@ -69,7 +69,7 @@ class ScalarTransform(Transform):
         for sym in sym0:
             j = 'xyzrs'.index(str(sym))
             m.append(mesh[j])
-        xj = sp.lambdify(sym0, dx)(*m)
+        xj = sp.lambdify(tuple(sym0), dx)(*m)
         self.input_array[...] = self.input_array*xj
         return
 
@@ -500,6 +500,19 @@ class TensorProductSpace(PFFT):
             return TensorProductSpace(comm, bases)
         return bases
 
+    def get_homogeneous(self, **kwargs):
+        """Return space with homogeneous boundary conditions, and otherwise
+        as self.
+
+        Parameters
+        ----------
+        kwargs : keyword arguments
+            Any arguments used in the creation of the different bases.
+
+        """
+        bases = tuple([base.get_homogeneous(**kwargs) for base in self.bases])
+        return TensorProductSpace(comm, bases)
+
     def dtype(self, forward_output=False):
         """Return datatype function space is planned for"""
         if hasattr(self, 'forward'):
@@ -942,7 +955,7 @@ class TensorProductSpace(PFFT):
         for sym in sym0:
             j = 'xyzrs'.index(str(sym))
             m.append(mesh[j])
-        xj = sp.lambdify(sym0, dx)(*m)
+        xj = sp.lambdify(tuple(sym0), dx)(*m)
         u *= xj
         return u
 
@@ -993,7 +1006,7 @@ class TensorProductSpace(PFFT):
         for axis, base in enumerate(self):
             if not base.family() == 'fourier':
                 axes.append(axis)
-            if base.family() == 'fourier' and (axis in msaxes):
+            if base.family() == 'fourier' and (axis in msaxes) and base.N > 1:
                 axes.append(axis)
         return axes
 
@@ -1647,7 +1660,7 @@ class BoundaryValues:
                         if sym == tt:
                             bci = bci.subs(tt, self.bc_time)
                     sym0 = bci.free_symbols
-                    lbci = sp.lambdify(sym0, bci, 'numpy')
+                    lbci = sp.lambdify(tuple(sym0), bci, 'numpy')
                     Yi = []
                     for sym in sym0:
                         k = 'xyzrs'.index(str(sym))
